
from django.conf.urls import url, include

# TODO: Add other patterns to allow introspection?

class Api(object):
    '''Helper class for registering many Publishers in one URL namespace'''
    def __init__(self, name):
        self.name = name
        self.children = {}

    def patterns(self, flat=False):
        urlpatterns = [
            url(r'^%s/' % name, include(child.patterns()))
            for name, child in self.children.items()
        ]
        if flat:
            return urlpatterns
        return [
            url(r'^%s/' % self.name, include(urlpatterns)),
        ]

    def register(self, child, name=None):
        if name is None:
            name = getattr(child, 'api_name', child.__class__.__name__.lower())
        if name in self.children:
            raise Warning('Publisher with name %s already registered: %r -> %r' % (
                name, self.children[name], child
            ))
        self.children[name] = child

APIS = {}

def register(name, *args):
    try:
        api = APIS[name]
    except KeyError:
        api = APIS[name] = Api(name=name)
    for resource in args:
        api.register(resource)
    return api

def autodiscover():
    from django.conf import settings
    from django.utils.importlib import import_module
    from django.utils.module_loading import module_has_submodule

    for app in settings.INSTALLED_APPS:
        mod = import_module(app)
        # Attempt to import the app's api module.
        try:
            import_module('%s.serialiser' % app)
        except:

            # Decide whether to bubble up this error. If the app just
            # doesn't have an admin module, we can ignore the error
            # attempting to import it, otherwise we want it to bubble up.
            if module_has_submodule(mod, 'serialiser'):
                raise

def patterns(flat=False):
    patterns = []
    for api in APIS.values():
        patterns.extend(api.patterns(flat=flat))
    return patterns





import json
from decimal import Decimal
import datetime

def digattr(obj, attr, default=None):
    '''Perform template-style dotted lookup'''
    steps = attr.split('.')
    for step in steps:
        try:    # dict lookup
            obj = obj[step]
        except (TypeError, AttributeError, KeyError):
            try:    # attribute lookup
                obj = getattr(obj, step)
            except (TypeError, AttributeError):
                try:    # list index lookup
                    obj = obj[int(step)]
                except (IndexError, ValueError, KeyError, TypeError):
                    return default
        if callable(obj):
            obj = obj()
    return obj

def undigattr(obj, attr, value):
    steps = attr.split('.')
    steps.reverse()
    last = steps.pop(0)
    for step in steps:
        try:
            obj = obj[step]
        except (TypeError, AttributeError, KeyError):
            try:    # attribute lookup
                obj = getattr(obj, step)
            except (TypeError, AttributeError):
                try:    # list index lookup
                    obj = obj[int(step)]
                except (IndexError, ValueError, KeyError, TypeError):
                    return default
        if callable(obj):
            obj = obj()
    setattr(obj, last, value)

from django.db.models import Model

class JSONEncoder(json.JSONEncoder):
    '''
    Same features as JSONEncoder, but not as a generator.
    '''
    def default(self, obj):
        if isinstance(obj, Decimal):
            return str(obj)
        if isinstance(obj, datetime.datetime):
            return '"' + obj.replace(microsecond=0).isoformat(' ') + '"'
        if isinstance(obj, datetime.date):
            return '"' + obj.isoformat() + '"'
        if hasattr(obj, '__iter__'):
            return list(obj)
        if isinstance(obj, Model):
            return unicode(obj)
        return super(JSONEncoder, self).default(obj)




from .utils import digattr, undigattr

class Field(object):

    def __init__(self, attribute=None, default=None, readonly=False,
        *args, **kwargs):
        self.attribute = attribute
        self.default = default
        self.readonly = readonly
        self.args = args
        self.kwargs = kwargs

    def _get_attrname(self, name):
        return self.attribute if self.attribute else name

    def deflate(self, name, obj, data, publisher=None):
        src = self._get_attrname(name)
        data[name] = digattr(obj, src, self.default)

    def inflate(self, name, data, obj, publisher=None):
        if self.readonly:
            return
        dest = self._get_attrname(name)
        try:
            setattr(obj, dest, data[name])
        except KeyError:
            pass

class SerialiserField(Field):
    def __init__(self, *args, **kwargs):
        super(SerialiserField, self).__init__(*args, **kwargs)
        self.serialiser = self.kwargs.pop('serialiser')

    def deflate(self, name, obj, data, publisher=None):
        src = self._get_attrname(name)
        val = digattr(obj, src, self.default)
        data[name] = self.serialiser.deflate_object(val)

    def inflate(self, name, obj, data, publisher=None):
        if self.readonly:
            return
        dest = self._get_attrname(name)
        try:
            val = self.serialiser.inflate_object(data[name])
        except KeyError:
            pass
        else:
           setattr(obj, dest, val)

class ManySerialiserField(Field):
    def __init__(self, *args, **kwargs):
        super(ManySerialiserField, self).__init__(*args, **kwargs)
        self.serialiser = self.kwargs.pop('serialiser')

    def deflate(self, name, obj, data, publisher=None):
        src = self._get_attrname(name)
        val = digattr(obj, src, self.default)
        data[name] = self.serialiser.deflate_list(iter(val), publisher=publisher)

    def inflate(self, name, obj, data, publisher=None):
        if self.readonly:
            return
        dest = self._get_attrname(name)
        try:
            val = self.serialiser.inflate_list(data[name], publisher=publisher)
        except KeyError:
            pass
        else:
            setattr(obj, dest, val)

